@@ -38,6 +38,8 @@
 #include <NetworkManagerQt/ActiveConnection>
 #include <NetworkManagerQt/Ipv4Setting>
 
+#include <libnm/nm-vpn-plugin-info.h>
+
 #if WITH_MODEMMANAGER_SUPPORT
 #include <ModemManagerQt/Manager>
 #include <ModemManagerQt/ModemDevice>
@@ -73,6 +75,16 @@ Handler::Handler(QObject *parent)
                                             QStringLiteral(AGENT_IFACE),
                                             QStringLiteral("registered"),
                                             this, SLOT(initKdedModule()));
+
+    QDBusConnection::sessionBus().connect(QStringLiteral(AGENT_SERVICE),
+                                            QStringLiteral(AGENT_PATH),
+                                            QStringLiteral(AGENT_IFACE),
+                                            QStringLiteral("secretsError"),
+                                            this, SLOT(secretAgentError(QString, QString)));
+
+    // Interval (in ms) between attempts to scan for wifi networks
+    m_wirelessScanRetryTimer.setInterval(2000);
+    m_wirelessScanRetryTimer.setSingleShot(true);
 }
 
 Handler::~Handler()
@@ -92,10 +104,24 @@ void Handler::activateConnection(const QString& connection, const QString& devic
         NetworkManager::VpnSetting::Ptr vpnSetting = con->settings()->setting(NetworkManager::Setting::Vpn).staticCast<NetworkManager::VpnSetting>();
         if (vpnSetting) {
             qCDebug(PLASMA_NM) << "Checking VPN" << con->name() << "type:" << vpnSetting->serviceType();
-            // get the list of supported VPN service types
+
+            bool pluginMissing = false;
+
+            // Check missing plasma-nm VPN plugin
             const KService::List services = KServiceTypeTrader::self()->query("PlasmaNetworkManagement/VpnUiPlugin",
                                                                               QString::fromLatin1("[X-NetworkManager-Services]=='%1'").arg(vpnSetting->serviceType()));
-            if (services.isEmpty()) {
+            pluginMissing = services.isEmpty();
+
+            // Check missing NetworkManager VPN plugin
+            if (!pluginMissing) {
+                GSList *plugins = nullptr;
+                plugins = nm_vpn_plugin_info_list_load();
+
+                NMVpnPluginInfo *plugin_info = nm_vpn_plugin_info_list_find_by_service(plugins, vpnSetting->serviceType().toStdString().c_str());
+                pluginMissing = !plugin_info;
+            }
+
+            if (pluginMissing) {
                 qCWarning(PLASMA_NM) << "VPN" << vpnSetting->serviceType() << "not found, skipping";
                 KNotification *notification = new KNotification("MissingVpnPlugin", KNotification::CloseOnTimeout, this);
                 notification->setComponentName("networkmanagement");
@@ -105,6 +131,7 @@ void Handler::activateConnection(const QString& connection, const QString& devic
                 notification->sendEvent();
                 return;
             }
+
         }
     }
 
@@ -142,7 +169,7 @@ void Handler::addAndActivateConnection(const QString& device, const QString& spe
 {
     NetworkManager::AccessPoint::Ptr ap;
     NetworkManager::WirelessDevice::Ptr wifiDev;
-    Q_FOREACH (const NetworkManager::Device::Ptr & dev, NetworkManager::networkInterfaces()) {
+    for (const NetworkManager::Device::Ptr &dev : NetworkManager::networkInterfaces()) {
         if (dev->type() == NetworkManager::Device::Wifi) {
             wifiDev = dev.objectCast<NetworkManager::WirelessDevice>();
             ap = wifiDev->findAccessPoint(specificObject);
@@ -257,7 +284,7 @@ void Handler::deactivateConnection(const QString& connection, const QString& dev
     }
 
     QDBusPendingReply<> reply;
-    Q_FOREACH (const NetworkManager::ActiveConnection::Ptr & active, NetworkManager::activeConnections()) {
+    for (const NetworkManager::ActiveConnection::Ptr &active : NetworkManager::activeConnections()) {
         if (active->uuid() == con->uuid() && ((!active->devices().isEmpty() && active->devices().first() == device) ||
                                                active->vpn())) {
             if (active->vpn()) {
@@ -278,7 +305,7 @@ void Handler::deactivateConnection(const QString& connection, const QString& dev
 
 void Handler::disconnectAll()
 {
-    Q_FOREACH (const NetworkManager::Device::Ptr & device, NetworkManager::networkInterfaces()) {
+    for (const NetworkManager::Device::Ptr &device : NetworkManager::networkInterfaces()) {
         device->disconnectInterface();
     }
 }
@@ -313,7 +340,7 @@ void Handler::enableBluetooth(bool enable)
         [this, enable] (QDBusPendingCallWatcher *watcher) {
             QDBusPendingReply<QMap<QDBusObjectPath, NMVariantMapMap> > reply = *watcher;
             if (reply.isValid()) {
-                Q_FOREACH (const QDBusObjectPath &path, reply.value().keys()) {
+                for (const QDBusObjectPath &path : reply.value().keys()) {
                     const QString objPath = path.path();
                     qCDebug(PLASMA_NM) << "inspecting path" << objPath;
                     const QStringList interfaces = reply.value().value(path).keys();
@@ -385,7 +412,7 @@ void Handler::removeConnection(const QString& connection)
     }
 
     // Remove slave connections
-    Q_FOREACH (const NetworkManager::Connection::Ptr &connection, NetworkManager::listConnections()) {
+    for (const NetworkManager::Connection::Ptr &connection : NetworkManager::listConnections()) {
         NetworkManager::ConnectionSettings::Ptr settings = connection->settings();
         if (settings->master() == con->uuid()) {
             connection->remove();
@@ -408,21 +435,43 @@ void Handler::updateConnection(const NetworkManager::Connection::Ptr& connection
     connect(watcher, &QDBusPendingCallWatcher::finished, this, &Handler::replyFinished);
 }
 
-void Handler::requestScan()
+void Handler::requestScan(const QString &interface)
 {
-    Q_FOREACH (NetworkManager::Device::Ptr device, NetworkManager::networkInterfaces()) {
+    for (NetworkManager::Device::Ptr device : NetworkManager::networkInterfaces()) {
         if (device->type() == NetworkManager::Device::Wifi) {
             NetworkManager::WirelessDevice::Ptr wifiDevice = device.objectCast<NetworkManager::WirelessDevice>();
             if (wifiDevice) {
+                if (!interface.isEmpty() && interface != wifiDevice->interfaceName()) {
+                    continue;
+                }
+                qCDebug(PLASMA_NM) << "Requesting wifi scan on device" << wifiDevice->interfaceName();
                 QDBusPendingReply<> reply = wifiDevice->requestScan();
                 QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(reply, this);
                 watcher->setProperty("action", Handler::RequestScan);
+                watcher->setProperty("interface", wifiDevice->interfaceName());
                 connect(watcher, &QDBusPendingCallWatcher::finished, this, &Handler::replyFinished);
             }
         }
     }
 }
 
+void Handler::scanRequestFailed(const QString &interface)
+{
+    if (m_wirelessScanRetryTimer.isActive()) {
+        return;
+    }
+    qCDebug(PLASMA_NM) << "Trying soon a new scan on" << interface;
+
+    emit wirelessScanTimerEnabled(false);
+
+    auto retryAction = [this,interface]() {
+        m_wirelessScanRetryTimer.disconnect();
+        requestScan(interface);
+    };
+    connect(&m_wirelessScanRetryTimer, &QTimer::timeout, this, retryAction);
+    m_wirelessScanRetryTimer.start();
+}
+
 void Handler::initKdedModule()
 {
     QDBusMessage initMsg = QDBusMessage::createMethodCall(QStringLiteral(AGENT_SERVICE),
@@ -432,6 +481,13 @@ void Handler::initKdedModule()
     QDBusConnection::sessionBus().send(initMsg);
 }
 
+void Handler::secretAgentError(const QString &connectionPath, const QString &message)
+{
+    // If the password was wrong, forget it
+    removeConnection(connectionPath);
+    emit connectionActivationFailed(connectionPath, message);
+}
+
 void Handler::replyFinished(QDBusPendingCallWatcher * watcher)
 {
     QDBusPendingReply<> reply = *watcher;
@@ -460,16 +516,17 @@ void Handler::replyFinished(QDBusPendingCallWatcher * watcher)
                 notification = new KNotification("FailedToRemoveConnection", KNotification::CloseOnTimeout, this);
                 notification->setTitle(i18n("Failed to remove %1", watcher->property("connection").toString()));
                 break;
-            case Handler::RequestScan:
-                /* INFO: Disabled for now as wifi scanning is now automatic
-                    notification = new KNotification("FailedToRequestScan", KNotification::CloseOnTimeout, this);
-                    notification->setTitle(i18n("Failed to request scan"));
-                */
-                break;
             case Handler::UpdateConnection:
                 notification = new KNotification("FailedToUpdateConnection", KNotification::CloseOnTimeout, this);
                 notification->setTitle(i18n("Failed to update connection %1", watcher->property("connection").toString()));
                 break;
+            case Handler::RequestScan:
+            {
+                const QString interface = watcher->property("interface").toString();
+                qCDebug(PLASMA_NM) << "Wireless scan on" << interface << "failed:" << error;
+                scanRequestFailed(interface);
+                break;
+            }
             default:
                 break;
         }
@@ -497,6 +554,10 @@ void Handler::replyFinished(QDBusPendingCallWatcher * watcher)
                 notification = new KNotification("ConnectionUpdated", KNotification::CloseOnTimeout, this);
                 notification->setText(i18n("Connection %1 has been updated", watcher->property("connection").toString()));
                 break;
+            case Handler::RequestScan:
+                qCDebug(PLASMA_NM) << "Wireless scan on" << watcher->property("interface").toString() << "succeeded";
+                emit wirelessScanTimerEnabled(true);
+                break;
             default:
                 break;
         }
